extern crate piston_window;
extern crate typemap;

pub use piston_window::*;
pub use typemap::{TypeMap, Key as TypeKey};

pub use std::collections::HashMap;

pub trait Entity {
    fn after_render(&mut self, globals: &mut TypeMap, after_render: &AfterRenderArgs, 
                    e: &PistonWindow, entity_cols: &mut HashMap<String, Vec<EntityObject>>) {}
    fn cursor(&mut self, globals: &mut TypeMap, cursor: &bool, 
              e: &PistonWindow, entity_cols: &mut HashMap<String, Vec<EntityObject>>) {}
    fn focus(&mut self, globals: &mut TypeMap, focus: &bool, 
             e: &PistonWindow, entity_cols: &mut HashMap<String, Vec<EntityObject>>) {}
    fn idle(&mut self, globals: &mut TypeMap, idle: &IdleArgs,
            e: &PistonWindow, entity_cols: &mut HashMap<String, Vec<EntityObject>>) {}
    fn press(&mut self, globals: &mut TypeMap, press: &Button, 
             e: &PistonWindow, entity_cols: &mut HashMap<String, Vec<EntityObject>>) {}
    fn release(&mut self, globals: &mut TypeMap, release: &Button, 
               e: &PistonWindow, entity_cols: &mut HashMap<String, Vec<EntityObject>>) {}
    fn render(&mut self, globals: &mut TypeMap, render: &RenderArgs, 
              e: &PistonWindow, entity_cols: &mut HashMap<String, Vec<EntityObject>>) {}
    fn resize(&mut self, globals: &mut TypeMap, resize: &[u32; 2], 
              e: &PistonWindow, entity_cols: &mut HashMap<String, Vec<EntityObject>>) {}
    fn text(&mut self, globals: &mut TypeMap, text: &String, 
            e: &PistonWindow, entity_cols: &mut HashMap<String, Vec<EntityObject>>) {}
    fn update(&mut self, globals: &mut TypeMap, update: &UpdateArgs, 
              e: &PistonWindow, entity_cols: &mut HashMap<String, Vec<EntityObject>>) {}
    fn clone_entity(&self) -> Box<Entity>;
}
pub struct EntityObject(pub Box<Entity>);
impl Clone for EntityObject {
    fn clone(&self) -> Self {
        EntityObject(self.0.clone_entity())
    }
}

pub struct EntityCollections;
impl TypeKey for EntityCollections { type Value = HashMap<String, Vec<EntityObject>>; }

pub struct Game {
    pub entity_collections: HashMap<String, Vec<EntityObject>>,
    pub globals: TypeMap,
    pub window: PistonWindow,
}
impl Game {
    pub fn new(window: PistonWindow) -> Self {
        Game {
            entity_collections: HashMap::new(),
            globals: TypeMap::new(),
            window: window
        }
    }
    pub fn run(mut self) {
        for e in self.window {
            let mut entity_collections = self.entity_collections.clone();
            for (name, entities) in self.entity_collections.iter_mut() {
                for entity in entities.iter_mut() {
                    let ref mut entity = entity.0;
                    if let Some(a) = e.after_render_args() {
                        entity.after_render(&mut self.globals, &a, &e, &mut entity_collections);
                    }
                    if let Some(a) = e.cursor_args() {
                        entity.cursor(&mut self.globals, &a, &e, &mut entity_collections);
                    }
                    if let Some(a) = e.focus_args() {
                        entity.focus(&mut self.globals, &a, &e, &mut entity_collections);
                    }
                    if let Some(a) = e.idle_args() {
                        entity.idle(&mut self.globals, &a, &e, &mut entity_collections);
                    }
                    if let Some(a) = e.press_args() {
                        entity.press(&mut self.globals, &a, &e, &mut entity_collections);
                    }
                    if let Some(a) = e.release_args() {
                        entity.release(&mut self.globals, &a, &e, &mut entity_collections);
                    }
                    if let Some(a) = e.render_args() {
                        entity.render(&mut self.globals, &a, &e, &mut entity_collections);
                    }
                    if let Some(a) = e.resize_args() {
                        entity.resize(&mut self.globals, &a, &e, &mut entity_collections);
                    }
                    if let Some(a) = e.text_args() {
                        entity.text(&mut self.globals, &a, &e, &mut entity_collections);
                    }
                    if let Some(a) = e.update_args() {
                        entity.update(&mut self.globals, &a, &e, &mut entity_collections);
                    }
                }
            }
        }
    }
}
